==> bsflite-0.80.tar.gz: bsflite-0.80/input.c <==
/**
 ** bsflite - bs-free AIM client
 **
 ** (C) 2003-2005 by Claudio Leite <leitec at leitec dot org>
 **
 ** NO WARRANTY. Read the file COPYING for more details.
 **/

#include "bsf.h"

extern char     inputbuf[512];
extern struct Conn *conn;
extern struct BuddyList *buddylist;
extern struct Waiting *waiting;
extern int      screen_lines;

/* PROTO */
void
parse_input(void)
{
	int             offset;

	if (strncasecmp(inputbuf, "q!", 2) == 0) {
		restore_tty();
		printf("\n");
		exit(-1);
	} else if (inputbuf[0] == 'm') {
		char           *msg, *temp, *temp2, *sn;
		char           *fullmsg;

		if (inputbuf[1] == ' ')
			temp2 = inputbuf + 2;
		else
			temp2 = inputbuf + 1;

		temp = strchr(temp2, ' ');

		if (temp == NULL) {
			printf("\nNo message to send.\n");
			return;
		}
		if (strlen(temp + 1) == 0) {
			printf("\nNo message to send.\n");
			return;
		}
		if (conn->netspeak_filter)
			msg = undo_netspeak(temp + 1);
		else
			msg = temp + 1;

		sn = malloc(temp - temp2 + 1);
		sn[temp - temp2] = 0;
		strncpy(sn, temp2, temp - temp2);
		fullmsg = malloc(strlen(msg) + strlen(SEND_FORMAT_BEGIN) + strlen(SEND_FORMAT_END) + 1);
		sprintf(fullmsg, "%s%s%s", SEND_FORMAT_BEGIN, msg, SEND_FORMAT_END);
		imcomm_im_send_message(conn->conn, sn, fullmsg, 0);
		free(fullmsg);
		eraseline();
#ifdef TIMESTAMPS
		addts();
		putchar(' ');
		offset = 13;
#else
		offset = 2;
#endif
		offset += strlen(sn) + 2;
#ifdef COLOR
		set_color(COLOR_OUTGOING_IM);
#endif
		printf("->%s", sn);
#ifdef COLOR
		set_color(0);
#endif
		printf(": ");
		wordwrap_print(msg, offset);

		if (conn->lastsn != NULL)
			free(conn->lastsn);
		conn->lastsn = strdup(sn);
		log_event(EVENT_IMSEND, sn, msg);
		free(sn);

		if (conn->netspeak_filter)
			free(msg);
		return;

	} else if (inputbuf[0] == 'w') {
		printf("\n** ");
		addts();
		printf(" %d buddies online:\n", conn->buddiesonline);

		if (inputbuf[1] == 'f')
			display_buddylist();
		else
			display_buddylist_sorted();

		return;
	} else if (inputbuf[0] == 'h' || inputbuf[0] == '?') {
		printf("\nbsflite commands:\n");
		printf("   h            : what you see right now\n");
		printf("   w            : show buddy list\n");
		printf("   ws           : show buddy list, sorted (default)\n");
		printf("   wf           : show buddy list, unsorted\n");
		printf("   l<sn>        : show last 10 lines from <sn>\n");
		printf("   ll           : show last 10 lines from last buddy\n");
		printf("   m<sn> <msg>  : send msg to sn\n");
		printf("   r<msg>       : reply to the last msg received/sent\n");
		printf("   a<sn>        : add buddy\n");
		printf("   d<sn>        : delete buddy\n");
		printf("   g<msg>       : set away message\n");
		printf("   g            : unset away message\n");
		printf("   G            : display pre-defined away messages\n");
		printf("   G<num>       : set pre-defined away message\n");
		printf("   i<sn>        : get profile and away message\n");
		printf("   A<sn>        : get away message\n");
		printf("   P<sn>        : get profile\n");
		printf("   b            : show away messages for all buddies\n");
		printf("   q!           : quit");
	} else if (inputbuf[0] == 'l') {
		if (inputbuf[1] == 'l' && inputbuf[2] == 0) {
			if (conn->lastsn != NULL) {
				show_log(20, conn->lastsn);
				return;
			}
		} else {
			show_log(20, inputbuf + 1);
			return;
		}
	} else if (inputbuf[0] == 'a') {
		char           *tmp;
		if (inputbuf[1] == ' ')
			tmp = inputbuf + 2;
		else
			tmp = inputbuf + 1;
		imcomm_im_add_buddy(conn->conn, tmp);
		printf("\n** Added buddy %s.\n", tmp);
		return;
	} else if (inputbuf[0] == 'd') {
		char           *tmp;
		if (inputbuf[1] == ' ')
			tmp = inputbuf + 2;
		else
			tmp = inputbuf + 1;

		imcomm_im_remove_buddy(conn->conn, tmp);
		printf("\n** Removed buddy %s.\n", tmp);
		remove_from_list(tmp);
		return;
	} else if (inputbuf[0] == 'g') {
		char           *tmp;
		if (conn->isaway) {
			conn->isaway = 0;
			free(conn->awaymsg);
			imcomm_set_unaway(conn->conn);
			printf("\n** You are no longer away.\n");
			return;
		} else {
			if (inputbuf[1] == ' ')
				tmp = inputbuf + 2;
			else
				tmp = inputbuf + 1;
			printf("\n** You are now away: %s\n", tmp);
			conn->isaway = 1;
			imcomm_set_away(conn->conn, tmp);
			conn->awaymsg = strdup(tmp);
			return;
		}
	} else if (inputbuf[0] == 'i') {
		char           *tmp;
		if (inputbuf[1] == ' ')
			tmp = inputbuf + 2;
		else
			tmp = inputbuf + 1;

		imcomm_request_awayprofile(conn->conn, tmp);
	} else if (inputbuf[0] == 'A') {
		char           *tmp;
		if (inputbuf[1] == ' ')
			tmp = inputbuf + 2;
		else
			tmp = inputbuf + 1;

		imcomm_request_awaymsg(conn->conn, tmp);
	} else if (inputbuf[0] == 'P') {
		char           *tmp;
		if (inputbuf[1] == ' ')
			tmp = inputbuf + 2;
		else
			tmp = inputbuf + 1;

		imcomm_request_profile(conn->conn, tmp);
	} else if (inputbuf[0] == 'I') {
		int             time;
		char           *tmp;

		if (inputbuf[1] == ' ')
			tmp = inputbuf + 2;
		else
			tmp = inputbuf + 1;

		time = atoi(tmp);

		set_idle_time(conn->conn, time);
	} else if (inputbuf[0] == 'b') {
		eraseline();
		printf("** Requesting away messages");
		fflush(stdout);
		showAwayMessages();
	} else if (inputbuf[0] == 'G') {
		int             msgnum = 0, count;
		struct AwayMessages *tv;

		if (conn->awaymsgs == NULL) {
			printf("\n** No away messages defined.\n");
			return;
		}
		if (inputbuf[1] == 0) {
			printf("\n** Defined messages:");
			for (tv = conn->awaymsgs, count = 0; tv != NULL; tv = tv->next, count++)
				printf("\n** [%02d] \"%s\"", count, tv->message);
		} else {
			msgnum = atoi(inputbuf + 1);
			for (tv = conn->awaymsgs, count = 0; tv != NULL; tv = tv->next, count++) {
				if (count == msgnum) {
					printf("\n** You are now away: %s", tv->message);
					conn->isaway = 1;
					imcomm_set_away(conn->conn, tv->message);
					conn->awaymsg = strdup(tv->message);
					break;
				}
			}
		}
	}
	printf("\n");
}

==> bsflite-0.81.tar.gz: bsflite-0.81/input.c <==
/**
 ** bsflite - bs-free AIM client
 **
 ** (C) 2003-2005 by Claudio Leite <leitec at leitec dot org>
 **
 ** NO WARRANTY. Read the file COPYING for more details.
 **/

#include "bsf.h"

extern char     inputbuf[512];
extern struct Conn *conn;
extern struct BuddyList *buddylist;
extern struct Waiting *waiting;
extern int      screen_lines;

/* PROTO */
void
parse_input(void)
{
	int             offset;

	if (strncasecmp(inputbuf, "q!", 2) == 0) {
		restore_tty();
		printf("\n");
		exit(-1);
	} else if (strncasecmp(inputbuf, "z!", 2) == 0) {
		printf("\n** Reloading away messages and profile.\n");
		set_profile(conn->conn);
		read_away_messages();
		return;
	} else if (inputbuf[0] == 'm') {
		char           *msg, *msg_strip, *temp, *temp2, *sn;
		char           *fullmsg;
		size_t          fullmsg_len;

		if (inputbuf[1] == ' ')
			temp2 = inputbuf + 2;
		else
			temp2 = inputbuf + 1;

		temp = strchr(temp2, ' ');

		if (temp == NULL) {
			printf("\n** No message to send.\n");
			return;
		}
		if (strlen(temp + 1) == 0) {
			printf("\nNo message to send.\n");
			return;
		}
		if (conn->netspeak_filter)
			msg = undo_netspeak(temp + 1);
		else
			msg = temp + 1;

		sn = malloc(temp - temp2 + 1);
		sn[temp - temp2] = 0;
		strncpy(sn, temp2, temp - temp2);
		fullmsg_len = strlen(msg) + strlen(SEND_FORMAT_BEGIN) + strlen(SEND_FORMAT_END) + 1;
		fullmsg = malloc(fullmsg_len + 1);
		snprintf(fullmsg, fullmsg_len, "%s%s%s", SEND_FORMAT_BEGIN, msg, SEND_FORMAT_END);
		imcomm_im_send_message(conn->conn, sn, fullmsg, 0);
		free(fullmsg);
		eraseline();

		if (conn->timestamps) {
			addts();
			putchar(' ');
			offset = 13;
		} else {
			offset = 2;
		}

		offset += strlen(sn) + 2;
#ifdef COLOR
		set_color(COLOR_OUTGOING_IM);
#endif
		printf("->%s", sn);
#ifdef COLOR
		set_color(0);
#endif
		printf(": ");
		msg_strip = strip_html(msg);
		wordwrap_print(msg_strip, offset);
		free(msg_strip);

		if (conn->lastsn != NULL)
			free(conn->lastsn);
		conn->lastsn = strdup(sn);
		log_event(EVENT_IMSEND, sn, msg);
		free(sn);

		if (conn->netspeak_filter)
			free(msg);
		return;

	} else if (inputbuf[0] == 'p') {
		char           *sn, *sendbuf, *pbuf;
		size_t          sendbuflen;
		ssize_t         pbuflen;
		uint16_t        maxmsgsize = imcomm_get_max_message_size(conn->conn);

		if (inputbuf[1] == ' ')
			sn = inputbuf + 2;
		else
			sn = inputbuf + 1;

		if (sn[0] == 0) {
			printf("\n** No recipient specified.\n");
			return;
		}
		printf("\n** Pasting to %s.\n", sn);
		printf("** Enter '.' on a line by itself to send,"
		       " Ctrl-X to cancel.");

		pbuf = malloc(maxmsgsize + 1);
		sendbuf = malloc(maxmsgsize + 1);
		memset(sendbuf, 0, maxmsgsize + 1);

		sendbuflen = 0;

		/*
		 * using fgets() produced strange results. had to resort to
		 * writing my own input stuff.
		 */

		while (1) {
			printf("\n-> ");
			fflush(stdout);

			pbuflen = bsf_getline(pbuf, maxmsgsize - 5);

			if (pbuflen == -1) {
				free(pbuf);
				free(sendbuf);
				printf("\n");
				return;
			}
			if (pbuflen == 1 && pbuf[0] == '.') {
				if (sendbuflen > 0) {
					imcomm_im_send_message(conn->conn, sn, sendbuf, 0);
				}
				break;
			}
			/*
			 * +4 is for "<br>"
			 */

			if (sendbuflen + pbuflen + 4 > maxmsgsize) {
				imcomm_im_send_message(conn->conn, sn, sendbuf, 0);
				memset(sendbuf, 0, maxmsgsize + 1);

				memcpy(sendbuf, pbuf, pbuflen);
				sendbuflen = pbuflen;
			} else {
#ifdef __OpenBSD__
				if (sendbuflen != 0) {
					strlcat(sendbuf, "<br>", maxmsgsize + 1);
					pbuflen += 4;
				}
				strlcat(sendbuf, pbuf, maxmsgsize + 1);
#else
				if (sendbuflen != 0) {
					strcat(sendbuf, "<br>");
					pbuflen += 4;
				}
				strcat(sendbuf, pbuf);
#endif
				sendbuflen += pbuflen;
			}
		}

		printf("\n");

		free(sendbuf);
		free(pbuf);
		return;
	} else if (inputbuf[0] == 'w') {
		printf("\n** ");
		addts();
		printf(" %d buddies online:\n", conn->buddiesonline);

		if (inputbuf[1] == 'f')
			display_buddylist();
		else
			display_buddylist_sorted();

		return;
	} else if (inputbuf[0] == 'h' || inputbuf[0] == '?') {
		printf("\nbsflite commands:\n");
		printf("   h            : what you see right now\n");
		printf("   w            : show buddy list\n");
		printf("   ws           : show buddy list, sorted (default)\n");
		printf("   wf           : show buddy list, unsorted\n");
		printf("   l<sn>        : show last 20 lines from <sn>\n");
		printf("   ll           : show last 20 lines from last buddy\n");
		printf("   m<sn> <msg>  : send msg to sn\n");
		printf("   r<msg>       : reply to the last msg received/sent\n");
		printf("   p<sn>        : paste to sn\n");
		printf("   a<sn>        : add buddy\n");
		printf("   d<sn>        : delete buddy\n");
		printf("   g<msg>       : set away message\n");
		printf("   g            : unset away message\n");
		printf("   G            : display pre-defined away messages\n");
		printf("   G<num>       : set pre-defined away message\n");
		printf("   i<sn>        : get profile and away message\n");
		printf("   A<sn>        : get away message\n");
		printf("   P<sn>        : get profile\n");
		printf("   z!		: reload profile and away messages\n");
		printf("   q!           : quit");
	} else if (inputbuf[0] == 'l') {
		if (inputbuf[1] == 'l' && inputbuf[2] == 0) {
			if (conn->lastsn != NULL) {
				show_log(20, conn->lastsn);
				return;
			}
		} else {
			show_log(20, inputbuf + 1);
			return;
		}
	} else if (inputbuf[0] == 'a') {
		char           *tmp;
		if (inputbuf[1] == ' ')
			tmp = inputbuf + 2;
		else
			tmp = inputbuf + 1;
		imcomm_im_add_buddy(conn->conn, tmp);
		printf("\n** Added buddy %s.\n", tmp);
		return;
	} else if (inputbuf[0] == 'd') {
		char           *tmp;
		if (inputbuf[1] == ' ')
			tmp = inputbuf + 2;
		else
			tmp = inputbuf + 1;

		imcomm_im_remove_buddy(conn->conn, tmp);
		printf("\n** Removed buddy %s.\n", tmp);
		remove_from_list(tmp);
		return;
	} else if (inputbuf[0] == 'g') {
		char           *tmp;
		if (conn->isaway) {
			conn->isaway = 0;
			free(conn->awaymsg);
			imcomm_set_unaway(conn->conn);
			printf("\n** You are no longer away.\n");
			return;
		} else {
			if (inputbuf[1] == ' ')
				tmp = inputbuf + 2;
			else
				tmp = inputbuf + 1;
			printf("\n** You are now away: %s\n", tmp);
			conn->isaway = 1;
			imcomm_set_away(conn->conn, tmp);
			conn->awaymsg = strdup(tmp);
			return;
		}
	} else if (inputbuf[0] == 'i') {
		char           *tmp;
		if (inputbuf[1] == ' ')
			tmp = inputbuf + 2;
		else
			tmp = inputbuf + 1;

		imcomm_request_awayprofile(conn->conn, tmp);
	} else if (inputbuf[0] == 'A') {
		char           *tmp;
		if (inputbuf[1] == ' ')
			tmp = inputbuf + 2;
		else
			tmp = inputbuf + 1;

		imcomm_request_awaymsg(conn->conn, tmp);
	} else if (inputbuf[0] == 'P') {
		char           *tmp;
		if (inputbuf[1] == ' ')
			tmp = inputbuf + 2;
		else
			tmp = inputbuf + 1;

		imcomm_request_profile(conn->conn, tmp);
	} else if (inputbuf[0] == 'I') {
		int             time;
		char           *tmp;

		if (inputbuf[1] == ' ')
			tmp = inputbuf + 2;
		else
			tmp = inputbuf + 1;

		time = atoi(tmp);

		imcomm_set_idle_time(conn->conn, time);
	} else if (inputbuf[0] == 'G') {
		int             msgnum = 0, count;
		struct AwayMessages *tv;

		if (conn->awaymsgs == NULL) {
			printf("\n** No away messages defined.\n");
			return;
		}
		if (inputbuf[1] == 0) {
			printf("\n** Defined messages:");
			for (tv = conn->awaymsgs, count = 0; tv != NULL; tv = tv->next, count++)
				printf("\n** [%02d] \"%s\"", count, tv->message);
		} else {
			msgnum = atoi(inputbuf + 1);
			for (tv = conn->awaymsgs, count = 0; tv != NULL; tv = tv->next, count++) {
				if (count == msgnum) {
					printf("\n** You are now away: %s", tv->message);
					conn->isaway = 1;
					imcomm_set_away(conn->conn, tv->message);
					conn->awaymsg = strdup(tv->message);
					break;
				}
			}
		}
	}
	printf("\n");
}

/* PROTO */
ssize_t
bsf_getline(char *str, int size)
{
	size_t          count = 0;
	char            ch;

	while (count < size - 1) {

#if defined(__MINGW32__) || defined(__DJGPP__)
		ch = getch();
#else
		read(fileno(stdin), &ch, 1);
#endif
		if (ch == '\n' || ch == '\r') {
			break;
		} else if (ch == 24) {
			/* Ctrl-X */
			return -1;
		} else if (ch == '\b' || ch == 127 || ch == 4) {
			count--;
			printf("\b \b");
			fflush(stdout);
		} else {
			str[count] = ch;
			count++;
			putchar(ch);
			fflush(stdout);
		}
	}

	str[count] = 0;

	return count;
}
